import lang.stride.*;
import java.util.*;
import greenfoot.*;

/**
 * Trieda reprezentuje smer. X rastie zľava doprava, Y rastie zhora dole.
 */
public class Smer
{
    private double x;
    private double y;
    private int uhol;

    /**
     * Vytvorí objekt ukazujúci daným smerom.
     */
    public Smer(double smerX, double smerY)
    {
        x = smerX;
        y = smerY;
        prepocitajUhol();
    }

    /**
     * Metóda prepočíta hodnotu atribútu uhol. Mala by sa volať po každej zmene atribútu x a y. 
     */
    private void prepocitajUhol()
    {
        /* keď je X 0, tak je otrebné pozrieť sa na Y. */
        if (x == 0) {
            if (y > 0) {
                /* idem hore*/
                uhol = 270;
            }
            else {
                /* idem dole*/
                uhol = 90;
            }
        }
        else {
            /* vypočítam si uhol od osi X v radiánoch , padá do (-PI/2 ; PI/2)*/
            double rad = Math.atan(y / x);
            /* pretransformujem ho do uhla v stupňoch, padá do (-90 ; 90)*/
            int deg = (int)Math.round(Math.toDegrees(rad));
            /* pretransofrumjem ho tak, aby bolo hore 270*/
            deg = deg + 360;
            /* teraz sa pozriem, či som naľavo od osi Y, ak áno, tak pridám 180*/
            if (y < 0) {
                deg = deg + 180;
            }
            /* nakoniec sa vrátim do 0 - 360*/
            uhol = deg % 360;
        }
    }

    /**
     * Vektor sa skráti na jednotkovú dĺžku.
     */
    public void normalizujSa()
    {
        double d = dlzka();
        x = x / d;
        y = y / d;
    }

    /**
     * Vráti dňlžku vektora.
     */
    public double dlzka()
    {
        return Math.sqrt(Math.pow(x, 2) + Math.pow(y, 2));
    }

    /**
     * Nastaví novú hodnotu X smeru. Zmena atribútu X je povolená výhradne pomocou tejto metódy.
     */
    private void zmenX(double noveX)
    {
        x = noveX;
        normalizujSa();
        prepocitajUhol();
    }

    /**
     * Nastaví novú hodnotu Y smeru. Zmena atribútu Y je povolená výhradne pomocou tejto metódy.
     */
    private void zmenY(double noveY)
    {
        y = noveY;
        normalizujSa();
        prepocitajUhol();
    }

    /**
     * Odrazí sa v X smere.
     */
    public void odrazHorizontalne()
    {
        zmenX( - x);
    }

    /**
     * Odrazí sa v Y smere.
     */
    public void odrazVertikalne()
    {
        zmenY( - y);
    }

    /**
     * Vráti uhol v stupňoch smeru voči ose X, uhol rastie v smere hodinových ručičiek.
     */
    public int getUhol()
    {
        return uhol;
    }

    /**
     * Pripočíta k sebe iný smer
     */
    public void pripocitaj(Smer smer)
    {
        x = x + smer.x;
        y = y + smer.y;
        prepocitajUhol();
    }
}
