import greenfoot.*;
import java.util.List;

public class Prvok extends Actor
{
    private static final int INTERVAL_PARENIA = 50;
    private static final int DOSPELOST = 250;
    private static final int STAROBA = 2000;
    private static final int POCET_PARENI_PRE_DELENIE = 3;
    private static final int PRAVDEPODOBNOST_DELENIA = 3;
    private static final int PRAVDEPODOBNOST_UMRTIA = 1;
    
    private Genom genom;
    private int casDoParenia;
    private Prvok poslednyPartner;
    private int vek;
    private int pocetPareni;
    
    public Prvok() {
        this(Genom.vytvorNahodnyGenom());
    }
    
    public Prvok(Genom genom) {
        this.genom = genom;
        this.casDoParenia = 0;
        this.poslednyPartner = null;
        this.vek = 0;
        this.pocetPareni = 0;
        
        this.nakresliSa();
    }
    
    public void act() 
    {
        this.vek++;
        
        if (this.umiera()) {
            this.umri();
        } else
        if (this.mozeSaRozdelit()) {
            this.rozdelSa();
        } else {       
            if (this.casDoParenia > 0) {
                this.casDoParenia--;
            }
            
            Prvok partner = this.najdiSiPartnera();
            if (partner != null) {
                this.sparujSa(partner);
            } else {
                this.pohniSa();
            }
            
            this.nakresliSa();
        }
    }    
    
    private void pohniSa() {
        if (this.mozeSaHybat()) {
            if (this.jeDospely()) {
                move(1);
            } else {
                move(2);
            }
            
            if (Greenfoot.getRandomNumber(100) < 50) {
                int uhol = 1 + Greenfoot.getRandomNumber(10);
                if (Greenfoot.getRandomNumber(100) < 50) {
                    this.turn(uhol);
                } else {
                    this.turn(-uhol);
                }
            }
        }
    }
    
    private Prvok najdiSiPartnera() {
        if (this.jePripravenyNaParenie()) {       
            List<Prvok> prvky = this.getIntersectingObjects(Prvok.class);
            Prvok najlepsiPrvok = null;
            double najlepsiaKrasa = 0;
            double mojaKrasa = this.krasa();
            
            for (Prvok prvok : prvky) {
                if (prvok != this.poslednyPartner && prvok.mozeSaParit(this)) {
                    double jehoKrasa = prvok.krasa();
                    if (jehoKrasa >= mojaKrasa && jehoKrasa - mojaKrasa <= 9*255 && jehoKrasa >= najlepsiaKrasa) {
                        najlepsiaKrasa = jehoKrasa;
                        najlepsiPrvok = prvok;
                    }
                }
            }
            
            return najlepsiPrvok;
        } else {
            return null;
        }
    }
    
    private void sparujSa(Prvok prvok) {
        this.poslednyPartner = prvok;
        
        Genom mojGenom = this.vytvorGenomNaParenie();
        Genom jehoGenom = prvok.vytvorGenomNaParenie();
        
        this.sparujGenom(jehoGenom);
        prvok.sparujGenom(mojGenom);
    }
    
    public void sparujGenom(Genom genomNaParenie) {
        this.casDoParenia = Prvok.INTERVAL_PARENIA;
        this.pocetPareni++;
        this.genom.sparujGeny(genomNaParenie);
    }
    
    private void nakresliSa() {        
        int vyska = this.vek >= Prvok.DOSPELOST ? Gen.VYSKA : Gen.VYSKA / 2;
        
        GreenfootImage platno = this.getImage();
        platno.clear();
        platno.scale(Genom.POCET_GENOV * Gen.SIRKA, Gen.VYSKA);
        
        for (int i = 0; i < Genom.POCET_GENOV; i++) {
            Gen gen = this.genom.getGen(i);
            Color farba = new Color(gen.getR(), gen.getG(), gen.getB());
            platno.setColor(farba);
            platno.fillRect(i * Gen.SIRKA, 0, Gen.SIRKA - 1, vyska - 1);
        }
        
        Color farba = this.jePripravenyNaParenie() ? Color.GREEN : Color.RED;
        platno.setColor(farba);
        platno.drawRect(0, 0, Genom.POCET_GENOV * Gen.SIRKA - 1, vyska - 1);
    }
    
    private void rozdelSa() {
        Prvok potomok1 = new Prvok(this.genom.vratKopiu());
        Prvok potomok2 = new Prvok(this.genom.vratKopiu());
        
        getWorld().addObject(potomok1, this.getX(), this.getY());
        getWorld().addObject(potomok2, this.getX(), this.getY());
        
        potomok1.turn(this.getRotation());
        potomok2.turn(this.getRotation() + 180);
        
        potomok1.move(this.getImage().getWidth() / 2 + 1);
        potomok2.move(this.getImage().getWidth() / 2 + 1);        
        
        this.umri();
    }
    
    public void umri() {
        getWorld().removeObject(this);
    }
    
    @Override
    public void setLocation(int x, int y)
    {
       
        while (x >= Miska.SIRKA_MISKY)
        {
            x -= Miska.SIRKA_MISKY;
        }
        while (x < 0)
        {
            x += Miska.SIRKA_MISKY;
        }
        while (y >= Miska.VYSKA_MISKY)
        {
            y -= Miska.VYSKA_MISKY;
        }
        while (y < 0)
        {
            y += Miska.VYSKA_MISKY;
        }
        
        super.setLocation(x, y);
    }
    
    public double krasa() {
        return this.genom.odlisnost(Miska.KRASNY_GENOM);
    }
    
    public boolean mozeSaParit(Prvok sKym) {
        return this.jePripravenyNaParenie() && sKym != this.poslednyPartner;
    }
    
    public boolean jePripravenyNaParenie() {
        return this.casDoParenia == 0 && this.vek >= Prvok.DOSPELOST;
    }
    
    public boolean mozeSaHybat() {
        return this.casDoParenia <= Prvok.INTERVAL_PARENIA / 2;
    }  
    
    public boolean mozeSaRozdelit() {
        return this.jeDospely() && this.pocetPareni >= Prvok.POCET_PARENI_PRE_DELENIE && Greenfoot.getRandomNumber(100) < Prvok.PRAVDEPODOBNOST_DELENIA; 
    }
    
    public boolean umiera() {
        int idealnyPocetPareni = (this.vek - Prvok.DOSPELOST) / Prvok.INTERVAL_PARENIA;
        int odchylka = idealnyPocetPareni - this.pocetPareni;
        return Greenfoot.getRandomNumber(100) < odchylka && Greenfoot.getRandomNumber(100) < Prvok.PRAVDEPODOBNOST_UMRTIA;
    }
    
    public boolean jeDospely() {
        return this.vek >= Prvok.DOSPELOST;
    }
    
    public Genom vytvorGenomNaParenie() {
        return this.genom.vratKopiu();
    }
}
