import lang.stride.*;
import java.util.*;
import greenfoot.*;
import java.awt.Color;

/**
 * Definuje ovocie, ktoré padá zhora dole a má svoju hodnotu.
 */
public class Ovocie extends OvladaciAktor
{
    private final int rychlost;
    private final int hodnota;

    /**
     * Inicialiizuje Jablko. Rýchlosť padania k zemi je definovaná parametrom.
     */
    public Ovocie(int paRychlost, int paHodnota)
    {
        rychlost = paRychlost;
        hodnota = paHodnota;
    }

    /**
     * Metíoda sprístupňuje košík v záhrade.
     */
    final public Kosik dajKosik()
    {
        Zahrada mojaZahrada = getWorldOfType(Zahrada.class);
        Kosik kosik = mojaZahrada.dajKosik();
        return kosik;
    }

    /**
     * Metóda sa vyvolá po dopadnutí ovocia do košíka.
     */
    protected void padDoKosika()
    {
        /* 1a - získanie inštancie košíka*/
        Kosik kosik = dajKosik();
        /* 1b - navýšenie skóre v košíku*/
        kosik.navysSkore(hodnota);
        kosik.znizKapacitu();
        /* 2 - odstránenie zo sveta*/
        odstranSaZoSveta();
    }

    /**
     * Metóda sa vyvolá po dopadnutí ovocia na zem.
     */
    protected void padNaZem()
    {
        odstranSaZoSveta();
    }

    /**
     * Volá sa automaticky v pravidelných intervaloch.
     */
    final public void konaj()
    {
        /* sekvancia "Spadni"*/
        int novaPoloha;
        novaPoloha = getY() + rychlost;
        zmenPolohuY(novaPoloha);
        /* Dopadlo na zem?*/
        if (jeNaDolnomOkraji()) {
            padNaZem();
        }
        else {
            /* Dotýka sa košíka?*/
            if (isTouching(Kosik.class)) {
                /* Sekvancia "Dopadni do košíka"*/
                padDoKosika();
            }
        }
    }

    /**
     * Vytvorí náhodné ovocie.
     */
    static public Ovocie vytvorOvocie()
    {
        /* vygenerujem  náhodný identifikátor ovocia - zatiaľ mám dve ovocia, maximum je teda 2*/
        int idOvocia = Greenfoot.getRandomNumber(6);
        /* vygenerujem náhodnú rýchlosť*/
        int nahodnaRychlost = 1 + Greenfoot.getRandomNumber(3);
        /* Pre každé ovocie pribudne vlastná vetva príkazu switch, v ktorej sa ovocie vygeneruje. Ak pribude ďalšie ovocie, toto je jediné miesto, ktoré bude potrebné upraviť.*/
        switch (idOvocia) {
            case 0 : {
                return  new Jablko(nahodnaRychlost);
            }
            case 1 : {
                return  new Jahoda(nahodnaRychlost);
            }
            case 2 : {
                return  new Citron(nahodnaRychlost);
            }
            case 3 : {
                return  new Hrozno(nahodnaRychlost);
            }
            case 4 : {
                return  new List(nahodnaRychlost);
            }
            case 5 : {
                return  new Kamen(nahodnaRychlost);
            }
            default : {
                return null;
            }
        }
    }
}
