import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)

/**
 * Trieda predstavuje notovu osnovu.
 * Obsahuje linky a klavesy.
 * Dokaze zahrat skladby.
 * 
 * @author Michal Varga 
 * @version 1.0
 */
public class NotovaOsnova extends World
{

    // premenne triedy urcuju velkost not a osnovy
    public static final int VYSKA_MEDZERY = 20;
    public static final int SIRKA_BRUSKA = 30;
    public static final int SIRKA_VLAJOCKY = 10;
    public static final int VYSKA_NOZICKY = 60;
    
    // premenne triedy urcuju velkost platna
    private static final int DLZKA = 800;
    private static final int VYSKA = 200;
    private static final int STRED = 100;
    
    // premenne triedy urcujuce suradnice tonov
    private static final int UROVEN_C3 = NotovaOsnova.STRED + 3 * NotovaOsnova.VYSKA_MEDZERY;
    private static final int UROVEN_D3 = NotovaOsnova.UROVEN_C3 - NotovaOsnova.VYSKA_MEDZERY / 2;
    private static final int UROVEN_E3 = NotovaOsnova.UROVEN_D3 - NotovaOsnova.VYSKA_MEDZERY / 2;
    private static final int UROVEN_F3 = NotovaOsnova.UROVEN_E3 - NotovaOsnova.VYSKA_MEDZERY / 2;
    private static final int UROVEN_G3 = NotovaOsnova.UROVEN_F3 - NotovaOsnova.VYSKA_MEDZERY / 2;
    private static final int UROVEN_A3 = NotovaOsnova.UROVEN_G3 - NotovaOsnova.VYSKA_MEDZERY / 2;
    private static final int UROVEN_H3 = NotovaOsnova.UROVEN_A3 - NotovaOsnova.VYSKA_MEDZERY / 2;
    
    private Skladba skladba; // skladba, ktoru prehra
    private int cas;
    
    /**
     * Vytvori prazdnu notovu osnovu.
     * 
     */
    public NotovaOsnova()
    {    
        super(NotovaOsnova.DLZKA, NotovaOsnova.VYSKA, 1); 
        
        GreenfootImage pozadie = this.getBackground();
        pozadie.setColor(Color.BLACK);
        pozadie.drawLine(0, NotovaOsnova.STRED - 2 * NotovaOsnova.VYSKA_MEDZERY, NotovaOsnova.DLZKA, NotovaOsnova.STRED - 2 * NotovaOsnova.VYSKA_MEDZERY);
        pozadie.drawLine(0, NotovaOsnova.STRED - 1 * NotovaOsnova.VYSKA_MEDZERY, NotovaOsnova.DLZKA, NotovaOsnova.STRED - 1 * NotovaOsnova.VYSKA_MEDZERY);
        pozadie.drawLine(0, NotovaOsnova.STRED, NotovaOsnova.DLZKA, NotovaOsnova.STRED);
        pozadie.drawLine(0, NotovaOsnova.STRED + 1 * NotovaOsnova.VYSKA_MEDZERY, NotovaOsnova.DLZKA, NotovaOsnova.STRED + 1 * NotovaOsnova.VYSKA_MEDZERY);
        pozadie.drawLine(0, NotovaOsnova.STRED + 2 * NotovaOsnova.VYSKA_MEDZERY, NotovaOsnova.DLZKA, NotovaOsnova.STRED + 2 * NotovaOsnova.VYSKA_MEDZERY);
        
        this.skladba = null;
        this.cas = 0;
        
        this.addObject(new Klaves(Ton.C3, "F1"), 50, this.kamZapisat(Ton.C3));
        this.addObject(new Klaves(Ton.D3, "F2"), 50, this.kamZapisat(Ton.D3));
        this.addObject(new Klaves(Ton.E3, "F3"), 50, this.kamZapisat(Ton.E3));
        this.addObject(new Klaves(Ton.F3, "F4"), 50, this.kamZapisat(Ton.F3));
        this.addObject(new Klaves(Ton.G3, "F5"), 50, this.kamZapisat(Ton.G3));
        this.addObject(new Klaves(Ton.A3, "F6"), 50, this.kamZapisat(Ton.A3));
        this.addObject(new Klaves(Ton.H3, "F7"), 50, this.kamZapisat(Ton.H3));
    }
    
    /**
     * Zapise notu do notovej osnovy.
     * 
     */
    public void zapisNotu(Nota nota) {
        if (!nota.jeZapisana()) {
            int pozicia = this.kamZapisat(nota.getTon());
            super.addObject(nota, NotovaOsnova.DLZKA - 50 + nota.getVychylenieStredX(), pozicia + nota.getVychylenieStredY()); 
            nota.zapis();
        }
    }
    
    /**
     * Na zaklade tonu vratu Y uroven, kam je potrebne zapisat prislusnu notu.
     * 
     */
    public int kamZapisat(Ton ton) {
        switch(ton) {
            case C3: return NotovaOsnova.UROVEN_C3;
            case D3: return NotovaOsnova.UROVEN_D3;
            case E3: return NotovaOsnova.UROVEN_E3;
            case F3: return NotovaOsnova.UROVEN_F3;
            case G3: return NotovaOsnova.UROVEN_G3;
            case A3: return NotovaOsnova.UROVEN_A3;
            case H3: return NotovaOsnova.UROVEN_H3;
            default: return -1;
        }
    }
    
    /**
     * Necha zahrat skladbu. Ak nejaka skladba hra, tak musi najskor skoncit.
     * 
     */
    public void zahrajSkladbu(Skladba skladba) {
        if (this.skladba != null) {
            System.out.println("Pockaj, kym dohra " + skladba.getNazov());
        } else {
            if (skladba.maTakty()) {
                this.skladba = skladba;
                this.cas = 0;
            } else {
                System.out.println(skladba.getNazov() + " nema ziadne noty");
            }
        }
    }
    
    public void act() {
        // ak ma priradenu skladbu, tak ju zahra.
        if (this.skladba != null) {
            if (this.skladba.maTakty()) {
                // vezmem noty z prveho taktu.
                Takt takt = this.skladba.pozriPrvyTakt();
                               
                // Tempo natahuje dobu trvania doby
                Nota nota = takt.dajNotuVCase(this.cas / this.skladba.getTempo());
                if (nota != null) {
                    this.zapisNotu(nota);
                }
                
                // posuniem cas
                this.cas++;
                
                // ak som presiel casom za trvanie taktu v tempe skladby, tak ho vyhodim
                if (this.cas >= takt.trvanieTaktu() * this.skladba.getTempo()) {
                    this.skladba.vyberPrvyTakt();
                    this.cas = 0;
                }
            } else {
                this.skladba = null;
            }
        }
    }
    
    /**
     * Zahra zakladnu toninu.
     * 
     */
    public final void zahrajToninu() {
        this.zahrajSkladbu(Skladba.vytvorSkladbuTonina());
    }
    
    /**
     * Zahra Pec nam spadla.
     * 
     */
    public final void zahrajPecNamSpadla() {
        this.zahrajSkladbu(Skladba.vytvorSkladbuPecNamSpadla());
    }
}
